﻿using System;
using System.Collections.Generic;
using System.Text;
using YAWD.Properties;
using YAWD.Utils;

namespace YAWD.BL.Providers
{
    class Randomizer : IProvider
    {
        public string URL { get; set; }
        public string Path { get; set; }
        public string FirstImage { get; set; }
        public string LastDwImage { get; set; }
        public DateTime LastUpdated { get; set; }
        public string LastId { get; set; }

        private bool _isCancelationPending;
        private double _current;
        private string _currentImage = string.Empty;
        private IList<string> _images;

        public double Current
        {
            get { return this._current; }
            set
            {
                this._current = value;
                OnProgressChanged(EventArgs.Empty);
            }
        }
        public string CurrentImage
        {
            get { return this._currentImage; }
            set
            {
                this._currentImage = value;
                OnCurrentImageChanged(EventArgs.Empty);
            }
        }
        public IList<string> Images
        {
            get { return this._images; }
            set
            {
                this._images = value;
                OnImagesChanged(EventArgs.Empty);
            }
        }

        public event EventHandler ProgressChanged;
        public event EventHandler CurrentImageChanged;
        public event EventHandler ImagesChanged;

        public IProvider Init()
        {
            return Util.ReadProviderSettings(this);
        }
        protected virtual void OnProgressChanged(EventArgs e)
        {
            if (this.ProgressChanged != null)
                this.ProgressChanged(this, e);
        }
        protected virtual void OnCurrentImageChanged(EventArgs e)
        {
            if (this.CurrentImageChanged != null)
                this.CurrentImageChanged(this, e);
        }
        protected virtual void OnImagesChanged(EventArgs e)
        {
            if (this.ImagesChanged != null)
                this.ImagesChanged(this, e);
        }

        public int getAll()
        {
            int start;
            if (int.TryParse(FirstImage, out start) == true)
                return getImages(start);
            else
                return 0;
        }
        private int getImages(int start)
        {
            this._isCancelationPending = false;

            int downloadedImage = 0;
            int end = getLatest();

            int nbrItems = end - start;
            double interleave = (double)100 / nbrItems;

            Settings settings = Settings.Default;
            string initPath;
            if (string.IsNullOrWhiteSpace(settings.WallPapersPath))
                initPath = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
            else
                initPath = settings.WallPapersPath;


            for (int i = start; i < end; i++)
            {
                //exit if the user cancels
                if (this._isCancelationPending == true)
                {
                    Util.SaveToConfigFile("LastUpdated", DateTime.Now.ToShortDateString(), this);
                    this.LastUpdated = DateTime.Now;
                    return downloadedImage;
                }

                Current += interleave;

                string url = URL + i;

                string tmpUrl = url + "/";
                string webPage = Util.GetHtmlPage(tmpUrl, Encoding.UTF8);
                if (webPage != null)
                {
                    string strParsing = @"attachment_";
                    int intTemp = webPage.IndexOf(strParsing);
                    if (intTemp > -1)
                    {
                        webPage = webPage.Substring(intTemp + strParsing.Length);
                        strParsing = @"href=""";
                        webPage = webPage.Substring(webPage.IndexOf(strParsing) + strParsing.Length);
                        strParsing = @"""";
                        string filepath = webPage.Substring(0, webPage.IndexOf(strParsing)).Trim();

                        string filename = filepath.Substring(filepath.LastIndexOf(@"/") + @"/".Length);

                        if (Util.GetImage(filepath, string.Empty,
                                                                            this.GetType().Name, filename,true) > 0)
                        {
                            downloadedImage++;
                            this.LastDwImage = i.ToString();
                            Util.SaveToConfigFile("LastDwImage", this.LastDwImage, this);
                            CurrentImage = this.LastDwImage;
                            Images = Util.LoadImage(Utils.Providers.Randomizer, initPath);
                        }
                    }
                }
            }
            this.LastUpdated = DateTime.Now;
            Util.SaveToConfigFile("LastUpdated", DateTime.Now.ToShortDateString(), this);
            return downloadedImage;
        }
        public int update()
        {
            int start;

            if (int.TryParse(LastDwImage, out start) == true)
                return getImages(start);
            else if (int.TryParse(FirstImage, out start) == true)
                return getImages(start);
            else
                return 0;
        }
        public void CancelAsync()
        {
            this._isCancelationPending = true;
        }

        private int getLatest()
        {
            string webPage = Util.GetHtmlPage("http://www.1920x1200.net/feed/", Encoding.UTF8);
            string strParsing = @"?p=";
            int intTemp = webPage.IndexOf(strParsing);
            if (intTemp > -1)
            {
                webPage = webPage.Substring(intTemp + strParsing.Length);
                strParsing = @"<";
                intTemp = webPage.IndexOf(strParsing);
                webPage = webPage.Substring(0, intTemp);

                int results;
                if (int.TryParse(webPage, out results) == true)
                    return results;

            }
            return 0;
        }
    }
}

