﻿using System;
using System.Collections.Generic;
using System.ServiceModel.Syndication;
using System.Text;
using YAWD.Properties;
using YAWD.Utils;
using YAWD.BL.Services;

namespace YAWD.BL.Providers
{
    class SkinsBe : IProvider
    {
        public string URL { get; set; }
        public string Path { get; set; }
        public string FirstImage { get; set; }
        public string LastDwImage { get; set; }
        public DateTime LastUpdated { get; set; }
        public string LastId { get; set; }

        private bool _isCancelationPending;
        private double _current;
        private string _currentImage = string.Empty;
        private IList<string> _images;

        public double Current
        {
            get { return this._current; }
            set
            {
                this._current = value;
                OnProgressChanged(EventArgs.Empty);
            }
        }
        public string CurrentImage
        {
            get { return this._currentImage; }
            set
            {
                this._currentImage = value;
                OnCurrentImageChanged(EventArgs.Empty);
            }
        }
        public IList<string> Images
        {
            get { return this._images; }
            set
            {
                this._images = value;
                OnImagesChanged(EventArgs.Empty);
            }
        }

        public event EventHandler ProgressChanged;
        public event EventHandler CurrentImageChanged;
        public event EventHandler ImagesChanged;

        public IProvider Init()
        {
            return Util.ReadProviderSettings(this);
        }
        protected virtual void OnProgressChanged(EventArgs e)
        {
            if (this.ProgressChanged != null)
                this.ProgressChanged(this, e);
        }
        protected virtual void OnCurrentImageChanged(EventArgs e)
        {
            if (this.CurrentImageChanged != null)
                this.CurrentImageChanged(this, e);
        }
        protected virtual void OnImagesChanged(EventArgs e)
        {
            if (this.ImagesChanged != null)
                this.ImagesChanged(this, e);
        }

        public int getAll()
        {
            return 0;
        }
        private int getImages(int start)
        {
            this._isCancelationPending = false;

            int downloadedImage = 0;
            int end = getLatest();

            int nbrItems = end - start;
            double interleave = (double)100 / nbrItems;
            IList<SyndicationItem> items = rssReader.getAll(new Uri(URL));

            Settings settings = Settings.Default;
            string initPath;
            if (string.IsNullOrWhiteSpace(settings.WallPapersPath))
                initPath = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
            else
                initPath = settings.WallPapersPath;

            int lastIndex;

            if (int.TryParse(this.CurrentImage, out lastIndex) == false)
                lastIndex = 0;

            IList<string> resolutions = ResolutionServices.GetEnabledResolutions();

            for (int i = end-1; i >= start; i--)
            {
                //exit if the user cancels
                if (this._isCancelationPending == true)
                {
                    Util.SaveToConfigFile("LastUpdated", DateTime.Now.ToShortDateString(), this);
                    return downloadedImage;
                }

                Current += interleave;

                string url = items[i].Id;
                string tmp = items[i].Id.Replace("/1024x768/", "");
                tmp = tmp.Substring(tmp.LastIndexOf("/") + "/".Length);

                int currentIndex;
                if (int.TryParse(tmp, out currentIndex) == false)
                    currentIndex = 1;

                if (currentIndex > lastIndex)
                {
                    foreach (string item in resolutions)
                    {
                        string tmpUrl = url.Replace("1024x768", item);
                        string webPage = Util.GetHtmlPage(tmpUrl, Encoding.UTF8);
                        if (webPage != null)
                        {
                            string strParsing = @"id=""wallpaper_image""";
                            int intTemp = webPage.IndexOf(strParsing);
                            if (intTemp > -1)
                            {
                                webPage = webPage.Substring(intTemp + strParsing.Length);
                                strParsing = @"src=""";
                                webPage = webPage.Substring(webPage.IndexOf(strParsing) + strParsing.Length);
                                strParsing = @"""";
                                string filepath = webPage.Substring(0, webPage.IndexOf(strParsing)).Trim();
                                string index = filepath.Substring(filepath.LastIndexOf("-") + "-".Length);
                                index = index.Substring(0, index.IndexOf("."));

                                string filename = filepath.Substring(filepath.LastIndexOf(@"/") + @"/".Length);

                                if (Util.GetImage(filepath, string.Empty,
                                                                                    this.GetType().Name, filename,false) > 0)
                                {
                                    downloadedImage++;
                                    this.LastDwImage = index;
                                    Util.SaveToConfigFile("LastDwImage", this.LastDwImage, this);
                                    CurrentImage = this.LastDwImage;
                                    Images = Util.LoadImage(Utils.Providers.SkinsBe, initPath);
                                    break;
                                }
                            }
                        }
                    }
                }
            }
            this.LastUpdated = DateTime.Now;
            Util.SaveToConfigFile("LastUpdated", DateTime.Now.ToShortDateString(), this);
            return downloadedImage;
        }
        public int update()
        {
            return getImages(0);
        }
        public void CancelAsync()
        {
            this._isCancelationPending = true;
        }

        private int getLatest()
        {
            return rssReader.getAll(new Uri(URL)).Count;
        }

    }
}

