﻿using System;
using System.Linq;
using System.Windows.Controls;
using Jint;
using YAWD.Utils;
using System.Collections.Generic;
using YAWD.Properties;
using System.Text;
using YAWD.BL.Services;

namespace YAWD.BL.Providers
{
    class wallBase : IProvider
    {
        public string URL { get; set; }
        public string Path { get; set; }
        public string FirstImage { get; set; }
        public string LastDwImage { get; set; }
        public DateTime LastUpdated { get; set; }
        public string LastId { get; set; }

        private bool _isCancelationPending;
        private double _current;
        private string _currentImage = string.Empty;
        private IList<string> _images;

        public double Current
        {
            get { return this._current; }
            set
            {
                this._current = value;
                OnProgressChanged(EventArgs.Empty);
            }
        }
        public string CurrentImage
        {
            get { return this._currentImage; }
            set
            {
                this._currentImage = value;
                OnCurrentImageChanged(EventArgs.Empty);
            }
        }
        public IList<string> Images
        {
            get { return this._images; }
            set
            {
                this._images = value;
                OnImagesChanged(EventArgs.Empty);
            }
        }

        public event EventHandler ProgressChanged;
        public event EventHandler CurrentImageChanged;
        public event EventHandler ImagesChanged;

        public IProvider Init()
        {
            return Util.ReadProviderSettings(this);
        }
        protected virtual void OnProgressChanged(EventArgs e)
        {
            if (this.ProgressChanged != null)
                this.ProgressChanged(this, e);
        }
        protected virtual void OnCurrentImageChanged(EventArgs e)
        {
            if (this.CurrentImageChanged != null)
                this.CurrentImageChanged(this, e);
        }
        protected virtual void OnImagesChanged(EventArgs e)
        {
            if (this.ImagesChanged != null)
                this.ImagesChanged(this, e);
        }

        public int getAll()
        {
            int start;
            if (int.TryParse(FirstImage, out start) == true)
                return getImages(start);
            else
                return 0;
        }
        private int getImages(int start)
        {
            this._isCancelationPending = false;

            int downloadedImage = 0;
            int end = getLatest();

            int nbrItems = end - start;
            double interleave = (double)100 / nbrItems;

            Settings settings = Settings.Default;
            string initPath;
            if (string.IsNullOrWhiteSpace(settings.WallPapersPath))
                initPath = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
            else
                initPath = settings.WallPapersPath;

            IList<string> resolutions = ResolutionServices.GetEnabledResolutions();

            for (int i = start; i < end; i++)
            {
                //exit if the user cancels
                if (this._isCancelationPending == true)
                {
                    Util.SaveToConfigFile("LastUpdated", DateTime.Now.ToShortDateString(), this);
                    this.LastUpdated = DateTime.Now;
                    return downloadedImage;
                }

                Current += interleave;

                string url = URL + i;

                string webPage = Util.GetHtmlPage(url, Encoding.UTF8);
                if (webPage != null && webPage.Contains("<title>Computer wallpapers / wallbase.cc</title>") == false)
                {
                    string strParsing = "line1";
                    int intTemp = webPage.IndexOf(strParsing);
                    if (intTemp > -1)
                    {

                        webPage = webPage.Substring(intTemp + strParsing.Length);
                        strParsing = @"white"">";
                        webPage = webPage.Substring(webPage.IndexOf(strParsing) + strParsing.Length);
                        strParsing = @"<";
                        string imageResolution = webPage.Substring(0, webPage.IndexOf(strParsing)).Trim();

                        bool goodResolution = resolutions.Any(resolution => resolution == imageResolution);

                        if (goodResolution == true)
                        {

                            strParsing = @"id=""bigwall""";
                            intTemp = webPage.IndexOf(strParsing);
                            if (intTemp > -1)
                            {
                                webPage = webPage.Substring(intTemp + strParsing.Length);
                                strParsing = @"src='";

                                intTemp = webPage.IndexOf(strParsing);
                                if (intTemp > -1)
                                {
                                    webPage = webPage.Substring(intTemp + strParsing.Length);
                                    strParsing = @"'";
                                }
                                else
                                {
                                    strParsing = @"src=""";
                                    webPage = webPage.Substring(webPage.IndexOf(strParsing) + strParsing.Length);
                                    strParsing = @"""";
                                }


                                string filepath = webPage.Substring(0, webPage.IndexOf(strParsing)).Trim();
                                strParsing = @"('";
                                filepath = filepath.Substring(filepath.IndexOf(strParsing) + strParsing.Length);
                                strParsing = @"')";
                                filepath = filepath.Substring(0, filepath.IndexOf(strParsing)).Trim();
                                filepath = uncrypte(filepath);
                                string filename = filepath.Substring(filepath.LastIndexOf(@"/") + @"/".Length);

                                if (Util.GetImage(filepath, string.Empty,
                                                  this.GetType().Name, filename, true) > 0)
                                {
                                    downloadedImage++;
                                    this.LastDwImage = i.ToString();
                                    Util.SaveToConfigFile("LastDwImage", this.LastDwImage, this);
                                    CurrentImage = this.LastDwImage;
                                    Images = Util.LoadImage(Utils.Providers.wallBase, initPath);
                                }
                            }
                        }
                    }
                }
                else
                    System.Threading.Thread.Sleep(5000);
            }
            this.LastUpdated = DateTime.Now;
            Util.SaveToConfigFile("LastUpdated", DateTime.Now.ToShortDateString(), this);
            return downloadedImage;
        }
        public int update()
        {
            int start;

            if (int.TryParse(LastDwImage, out start) == true)
                return getImages(start);
            else if (int.TryParse(FirstImage, out start) == true)
                return getImages(start);
            else
                return 0;
        }
        public void CancelAsync()
        {
            this._isCancelationPending = true;
        }
        private int getLatest()
        {
            string webPage = Util.GetHtmlPage("http://wallbase.cc/search", Encoding.UTF8);
            if (string.IsNullOrWhiteSpace(webPage))
            {
                int id;
                int.TryParse(LastId, out id);
                return id;
            }

            string strParsing = @"drg_thumb";
            int intTemp = webPage.IndexOf(strParsing);
            if (intTemp > -1)
            {
                webPage = webPage.Substring(intTemp + strParsing.Length);
                strParsing = @"""";
                intTemp = webPage.IndexOf(strParsing);
                webPage = webPage.Substring(0, intTemp);

                int results;
                if (int.TryParse(webPage, out results) == true)
                    return results;

            }
            return 0;
        }

        private string uncrypte(string crypted)
        {
            const string script = @"function B(a){var b='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';var c,d,e,f,g,h,i,j,k=0,l=0,m='',n=[];if(!a){return a}a+='';do{f=b.indexOf(a.charAt(k++));g=b.indexOf(a.charAt(k++));h=b.indexOf(a.charAt(k++));i=b.indexOf(a.charAt(k++));j=f<<18|g<<12|h<<6|i;c=j>>16&255;d=j>>8&255;e=j&255;if(h==64){n[l++]=String.fromCharCode(c)}else if(i==64){n[l++]=String.fromCharCode(c,d)}else{n[l++]=String.fromCharCode(c,d,e)}}while(k<a.length);m=n.join('');return m} return B(cypted);";

            var result = new JintEngine();
            result.SetParameter("cypted", crypted);
            var toto=result.Run(script);

            return toto.ToString();

        }
    }
}
